import re
import subprocess


def _parse_names(data: str) -> list[str]:
    """
    Parses names from a multi-line string where each line contains a name and other details.

    Parameters:
    data (str): A multi-line string containing names and other details.

    Returns:
    list: A list containing the parsed names.
    """
    # Split the string into lines
    lines = data.strip().split("\n")

    # Skip the header line
    lines = lines[1:]

    # Regex pattern to match the name at the beginning of each line
    pattern = r"^\S+"

    # Extract names using regex
    names = [
        re.match(pattern, line).group(0) for line in lines if re.match(pattern, line)
    ]

    return names


def _run_commandline(command: str) -> str:
    """
    Runs a commandline command and returns the output.

    Parameters:
    command (str): A string containing the command to be run.

    Returns:
    str: A string containing the output from the command.
    """

    command = command.split(" ")

    # Run the command
    result = subprocess.run(command, capture_output=True)

    # Get the output
    output = result.stdout.decode("utf-8")

    return output


def list_models() -> list[str]:
    """
    Lists all available OLLAMA models.

    Parameters:
    None

    Returns:
    None
    """
    # Run the command, and assert ollama is installed

    try:
        data = _run_commandline("ollama list")
    except FileNotFoundError as e:
        raise FileNotFoundError(
            "Unable to find Ollama command. Please install OLLAMA first. See https://ollama.ai for more details.")

    # Parse the names
    names = _parse_names(data)

    # assert len(names) < 0, "No models found. Please install OLLAMA first."


    return sorted(names)


OLLAMA_MODELS = list_models()

if __name__ == "__main__":
    # get list of names
    names = list_models()
    print(names)



import streamlit as st
from llama_index.llms import Ollama
from llama_index.llms import ChatMessage
# https://docs.llamaindex.ai/en/stable/examples/llm/ollama.html

from list_ollama_models import OLLAMA_MODELS



def st_ollama(model_name, user_question, chat_history_key):

    llm_ = Ollama(model=model_name)
    
    if chat_history_key not in st.session_state.keys():
        st.session_state[chat_history_key] = []

    print_chat_history_timeline(chat_history_key)
        
    # run the model
    if user_question:
        st.session_state[chat_history_key].append({"content": f"{user_question}", "role": "user"})
        with st.chat_message("question", avatar="🧑‍🚀"):
            st.write(user_question)

        # with st.spinner("Thinking ..."):
        messages = [ChatMessage(content=message["content"], role=message["role"]) for message in st.session_state[chat_history_key]]

        response = llm_.stream_chat(messages)
        
        # streaming response
        with st.chat_message("response", avatar="🤖"):
            chat_box = st.empty()
            response_message = ""
            for chunk in response:
                response_message += chunk.delta
                chat_box.write(response_message)        

        st.session_state[chat_history_key].append({"content": f"{response_message}", "role": "assistant"})
        
        return response_message


def print_chat_history_timeline(chat_history_key):
    for message in st.session_state[chat_history_key]:
        role = message["role"]
        if role == "user":
            with st.chat_message("user", avatar="🧑‍🚀"): 
                question = message["content"]
                st.markdown(f"{question}", unsafe_allow_html=True)
        elif role == "assistant":
            with st.chat_message("assistant", avatar="🤖"):
                st.markdown(message["content"], unsafe_allow_html=True)


# -- helpers --
def page_config():
    st.set_page_config(layout="wide", page_title="Ollama Chat", page_icon="🦙")
    footer = """<div style="position: fixed; left: 0; bottom: 0; background-color: #004d4d; color: white; padding: 10px 20px; border-top-right-radius: 10px; box-shadow: 2px 2px 5px grey; font-size: 10px; opacity: 0.5;">
                Source code <a href="https://github.com/iamaziz/st_ollama" style="color: white; text-decoration: none; font-weight: bold;">here</a>
                <br>
                <span style="font-size: 10px;">2023 By Aziz Alto</span>
        </div>"""
    st.sidebar.markdown(footer, unsafe_allow_html=True)


def assert_models_installed():
    if len(OLLAMA_MODELS) < 1:
        st.sidebar.warning("No models found. Please install at least one model e.g. `ollama run llama2`")
        st.stop()


if __name__ == "__main__":

    page_config()
    
    st.sidebar.title("Ollama Chat 🦙")
    llm_name = st.sidebar.selectbox("Choose Agent", [""] + OLLAMA_MODELS)
    
    assert_models_installed()
    
    if not llm_name: st.stop()

    conversation_key = f"model_{llm_name}"
    prompt = st.chat_input(f"Ask '{llm_name}' a question ...")

    st_ollama(llm_name, prompt, conversation_key)
    
    if st.session_state[conversation_key]:
        clear_conversation = st.sidebar.button("Clear chat")
        if clear_conversation:
            st.session_state[conversation_key] = []
            st.rerun()




import subprocess
import os

def run_streamlit():
    # Get the directory where the package is installed
    dir_path = os.path.dirname(os.path.realpath(__file__))
    chatbot_path = os.path.join(dir_path, 'chatbot.py')

    # Run the command
    try:
        subprocess.run(["streamlit", "run", chatbot_path])
    except KeyboardInterrupt:
       print("Shutting down the Streamlit app...")
       exit(0)


from setuptools import setup, find_packages

HOME_URL = "https://github.com/iamaziz/st_ollama"

setup(
    name="st_ollama",
    version="0.1.1",
    author="Aziz Alto",
    author_email="iamaziz.alto@gmail.com",
    description="A Streamlit chatbot app integrating Ollama LLMs",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url=HOME_URL,
    packages=find_packages(),
    install_requires=[
        "streamlit",
        "llama_index",
    ],
    package_data={'st_ollama': ['chatbot.py']},
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "ollachat=ollachat.cli:run_streamlit",
        ],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)
